	LIST P=16C71
;software to sample data at RA1 and output it serially at 115.2kbaud to a PC

INCLUDE "picreg.equ"

;port A is used as follows: bit 0 - not used (tie to ground
;                           bit 1 - analogue input
;                           bit 2 - not used
;                           bit 3 - Vref
;port B is used:            bit 7 - RS232 output (TTL Low = RS232 +ve using interface)
;                           bits 2 & 3 Sample rate select
;                           other bits not used (set to inputs)

;equates here

LAST_RDG        equ     0x0c
TEMP            equ     0x0d
DEL_REG         equ     0x0e
S1              equ     0x10
S2              equ     0x11
S3              equ     0x12
S4              equ     0x13
ILO             equ     0x14    
IHI             equ     0x15
QLO             equ     0x16
QHI             equ     0x17

;code starts here

	org     PIC71
boot    
	nop
	clrw
	movwf   INTCON          ;disable interrupts
	goto    startup         ;jump to main code
ints    
	retfie                  ;interrupt service routine code

startup 
	bsf     STATUS,RP0      ;ram page 1
	movlw   b'11111111'     ;
	movwf   TRISA           ;set port a as inputs
	movlw   b'01111111'     ;set Port B:7 as output, PB6-0 input
	movwf   Port_B
	bcf     STATUS,RP0      ;ram page 0
	movlw   b'10000000'     ;set RS232 to quiescent state
	movwf   Port_B

;setup the A-D here
analogue
	movlw   b'01001001'     ;Tad = fosc/8, ch1, A/D on
	movwf   ADCON0
	bsf     STATUS,RP0
	movlw   b'00000010'     ;set RA0,1 as analogue, RA2,3 as digital
	movwf   ADCON1
	bcf     STATUS,RP0
	clrf    LAST_RDG
	btfsc   Port_B , 2      ;RB2 & RB3 at 0 = I/Q sampling at 1kHz
	goto    loopreal
	btfss   Port_B , 3
	goto    loopIQ
;-----------------------------------------------------------------------
loopreal                    ;loop should be 100/140/400 cycles long
	bsf     ADCON0,GO       ;set conversion going whilst we send last sample
	call    send            ;call + send = 85 cycles
;---------------------------------------------------------------------------
				  ;Extra delay routines in here
	btfsc   Port_B , 3        ;B3 = 1, 100/140us  B3 = 0, 400/1000us
	goto    FastRates
	btfsc   Port_B , 2        ;test for I/Q or 400us
	goto    Del400
	goto    loopIQ

Del400
	movlw    d'99'         ;extra 300us delay to make 400
	movwf    DEL_REG
Loop400
	decfsz   DEL_REG
	goto Loop400
	nop
	goto CarryOn

FastRates
	btfsc Port_B , 2        ;test for 100 or 140us
	goto  CarryOn
	
	movlw    d'13'          ;extra 40us to make 140
	movwf    DEL_REG
Loop40
	decfsz   DEL_REG        ;delay for basic loop = 3 x N + 1
	goto Loop40
	nop
;----------------------------------------------------------------------------
CarryOn
	nop                     ;nops to make up 100 cycles for fastest rate
	nop
	nop
	nop
	movf    ADRES,0    
	movwf   LAST_RDG  
	goto    loopreal        ;do this forever or until RB2/3 = 00
;----------------------------------------------------------------------------
loopIQ                       
	bsf     ADCON0,GO       ;set conversion of S1 going
	clrf    IHI             ;whilst sending previous data. Need
	movf    S1, 0           ;250us between ADCON0 instructions
	movwf   ILO
	movf    S2, 0
	addwf   ILO
	btfsc   STATUS, CARRY
	incf    IHI
	movf    S3, 0
	subwf   ILO
	btfss   STATUS, CARRY
	decf    IHI
	movf    S4, 0
	subwf   ILO
	btfss   STATUS, CARRY
	decf    IHI
	movf    IHI, 0
	andlw   b'00000011'     ;IHI/ILO contains 10 bit 2s complement
	movwf   IHI             ;S1 + S2 - S3 - S4
	nop
	nop
	nop
	nop
	clrf    QHI
	movf    S1, 0
	movwf   QLO
	movf    S4, 0
	addwf   QLO
	btfsc   STATUS, CARRY
	incf    QHI
	movf    S2, 0
	subwf   QLO
	btfss   STATUS, CARRY
	decf    QHI
	movf    S3, 0
	subwf   QLO
	btfss   STATUS, CARRY
	decf    QHI             
	movf    QHI, 0          ;QHI/QLO contains 10 bit 2s complement
	iorlw   b'11111100'     ;S1 - S2 - S3 + S4  with MSBs set as Q marker
	movwf   QHI
	
	btfsc   Port_B , 2      ;Test for change of sampling rate to real
	goto    loopreal
	btfsc   Port_B , 3
	goto    loopreal
	nop                
	movf    IHI, 0
	call    send            ;transmit IHI character
	movf    ADRES, 0
	movwf   S1
	movlw   d'22'           ;115us delay
	call    delay           ;to make up to 250us

	bsf     ADCON0,GO       ;Trigger S2 conversion
	movf    ILO, 0
	call    send            ;transmit ILO character
	movf    ADRES, 0        ;send is 85us so OK to read A/D now
	movwf   S2
	movlw   d'31'
	call    delay           ;to make up to 250us
	
	bsf     ADCON0,GO       ;Trigger S3 conversion
	movf    QHI, 0
	call    send            ;
	movf    ADRES, 0
	movwf   S3
	movlw   d'31'
	call    delay           ;to make up to 250us
	
	bsf     ADCON0,GO       ;Trigger S4 conversion
	movf    QLO, 0
	call    send            ;
	movf    ADRES, 0
	movwf   S4
	movlw   d'30'
	call    delay           ;to make up to 250us,less following instructions
	nop
	nop
	nop
	goto    loopIQ
;------------------------------------------------------------------------
send                        ;this routine 85us including call
	movwf   LAST_RDG        ;Enter with data to be sent in W register
	movlw   b'00000000'     ;1 start bit
	movwf   Port_B          ;bit time is 8.67us, so we will necessarily have +/- 0.5us jitter
	nop                     ;this bit 9us
	nop
	nop
	nop
	clrf    TEMP            ;clear output reg (not absolutely necessary)
	rrf     LAST_RDG,1      ;rotate bottom bit to carry
	rrf     TEMP,1          ;rotate carry into top bit
	movf    TEMP,0          ;get ready to output
	movwf   Port_B          ;and output bit
	nop                     ;this bit 9us (bit 0)
	nop
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 8us (bit 1)
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 9us (bit 2)
	nop
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 9us (bit 3)
	nop
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 8us (bit 4)
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 9us (bit 5)
	nop
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 9us (bit 6)
	nop
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 8us (bit 7)
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movlw   b'10000000'
	movwf   Port_B          ;this bit stop bit generated elsewhere
	return
;---------------------------------------------------------------------------
delay
	movwf   DEL_REG         ;Enter with N in W register
deloop  
	nop                     ;Total delay = 5.N + 3 (+ 2 to call = 5.N + 5)
	nop
	decfsz  DEL_REG
	goto    deloop
	nop
	return
;------------------------------------------------------------------------
	END

